# Rocketship theme: Rocketship Flex

Component-based Drupal theme for use with Dropsolid Rocketship install profile, modules and other components.

## Dependencies

### Local development

- Node: 12.x
- NPM: 6.x (or whatever version comes with 12)
- a recent version of gulp-cli

We don't recommend downgrading to node 8, due to lack of support.
[https://blog.risingstack.com/update-nodejs-8-end-of-life-no-support/](Version 8 dropped maintenance support in December 2019)
Version 10 should work but support will stop in April 2021.
At the time of writing, version 12 is the latest stable release.

If you need to upgrade (or downgrade), here is some info:
- Install and switch to the node version you will be using (we recommend using NVM)
- throw away the package.lock files
- throw away the node_modules folder
- change the version check in package.json with the version you will be working on
- DO NOT USE AN UNEVEN NODE VERSION!!! Only the even ones are stable.
- run `npm install`
- rebuild node-sass: `npm rebuild node-sass`

**Examples:**
Current version:
  ````
  "node": ">=10.1.0 <11.0.0 || >=12.1.0 <13.0.0"
  ````
  ````
Upgrade to 14:
  ````
  "node": ">=14.0.0 <15.0.0"
  ````

### Setup

Before you do anything else, uninstall (disable) the front-end themes you are NOT using.
Having multiple themes enabled, seems to break some the inheritance rules (eg. libraries or components of same name get picked up from the wrong theme)
This issue describes the problem and provides a patch if you need it: https://www.drupal.org/project/components/issues/3027672


**INSTALL NVM TO MANAGE YOUR NODE VERSIONS!**

The basis of our local development, is Node + NPM. We need it to run Gulp tasks to make our life easier.
Depending on how old your project is, you will need to quickly and easily switch Node versions.
This is because some versions of Gulp (and dependencies like sass-build) are not compatible or need updated builds.

Long story short: you need to install NVM so you can switch using a simple command & so we can enforce locked versions of NPM.
Follow the instructions set here:
* https://redmine.dropsolid.com/issues/84063
* Or here: https://michael-kuehnel.de/node.js/2015/09/08/using-vm-to-switch-node-versions.html

Refer to the theme's readme file to see what version you need to run in order to work with that theme.
Unfortunately, most projects pre-06/2019 don't have this info, but these pointers might help you:
- if it contains Gulp 3 and no info in the readme, it is very likely you'll need to switch to NPM 6
- if it contains Gulp 4 and no info in the readme, either NPM 6 and NPM 8 should work but you might need to change some things:
  - Pay attention to the output when running `npm install`. If it gives you a warning about node-sass, you need to rebuild it. The output should contain the right command to do so.
- themes after 06/2019 should contain a version lock for NPM + info in the 'Requirements'-section of the theme's readme file on what NPM version to use.

### Drupal stuff (possibly already included by way of Rocketship Core)

- [Drupal Components Library](https://www.drupal.org/project/components)
- [Unified Twig Extentions](https://github.com/drupal-pattern-lab/unified-twig-extensions)
- [styleguide module]((https://www.drupal.org/project/styleguide))
- responsive_image module (part of Drupal core)

### Optional:
- styleguide module and rocketship styleguide module

## What the theme does

- Templates, CSS and JS is component-based
- Converts Sass (with globbing) into CSS
- Can provide Sourcemaps for CSS and JS
- Generates favicons
- Generates custom icon font or icon sprite
- has better responsive tables
- can override colors for the Rocketship Paragraphs background-color palette
- adds styling for Rocketship content types

## Settings

Your active theme has settings over at `admin/appearance/settings/themename`
Be sure to check those out before searching the Sass and JS files.
E.g. there is a setting that can switch the language menu from a dropdown to an inline list
E.g. the Flex theme has a setting to make a fixed header

## Libraries & other third party stuff

- **Fontawesome:** icons made easy, integrated with a theme mixin and variables
  - We have included `Fontawesome 5 Free`, but if you have bought the `Pro` version, you can easily switch to that by:
    - replacing the font-files in your generated theme's `fonts`-folder
    - change any font-file names (if needed) in `THEME/components/00-theme/00-base/02-fonts/_fonts.scss`
    - filling in the `fontawesome-version` variable with `pro` (instead of the default `free`) in `THEME/components/00-theme/00-base/00-config/00-variables/_vars-icons.scss`

- **Styleguide:** Drupal module to generate a styleguide to theme (if dropsolid_styleguide added & enabled)
Located at `/admin/appearance/styleguide`


## Setup

- **Don't change the name of this theme after it has been generated!** It will break things.
- Check if your npm and node are up-to-date. In your command line, run `npm -v` and `node -v`. The versions should match the requirements.
- In your command line, navigate to your theme folder (*/docroot/sites/all/themes/custom/[theme]*) and run `npm install`. You only need to run this command once, when you install the theme. It will install the node modules for gulp to work.
- if your project has a docroot-folder (and uses git), a pre-install script will run and try to add a git pre-commit hook that helps detect linting issues
- In your Drupal settings, make sure your child theme is activated and set as default.

Your theme is now ready to be used!


## Technology

- **Gulp** is used as a task manager (eg. compile CSS, make favicons, …)
- **npm** is used to manager the node modules for Gulp, and if needed, run
pre or post scripts (see package.json)
- **Sass** + linting is used to write better CSS.
- **JS & jQuery**: used for all manner of interactivity, such as the mobile
navigation.
 The base theme replaces the default jQuery version in the front-end with 3.3.x
- **Modernizer**: Useful for feature-detection (css or js properties support in
order to load fallbacks). The base theme replaces the default with version 3.6.x
and loads it for anonymous users as well.


## Anatomy the theme

- .theme file
- yml-files
- components folder
- config
- gulp
- css and js
- fonts, images, favicons, icons, …

### rocketship_theme_flex.theme

Various theme functions (hooks). The most important ones being to:

- **Add theme settings:** `HOOK_form_system_theme_settings_alter` and
`HOOK_preprocess_page`
  Theme settings are defined and saved in drupal settings so they are
  available in your JS files (using the drupalSettings object) and in your
  theme's settings page. An example of this being used, would be the language
  menu and fixed header (Flex & Demo only).
- **Add background-colors for paragraphs:**
  `_rocketship_paragraphs_bg_color_options_list`
  This is where you need to add the extra colors that the simple header and
  other paragraphs use.
  Works with `00-theme/admin.style.scss` to build the preview in the form
  display.
- **Load libraries:**
  `HOOK_preprocess_page`
  For conditionally loading libraries. DO NOT LOAD ALL YOUR LIBRARIES HERE!!!
  Only for exeptional cases. Eg. loading the feature libraries happens here.

Don't forget that the child theme inherits theme functions defined in the
parent 'Base' theme.

Eg. for turning responsive tables on or off (selectively):
`$variables['table']['#responsive'] = FALSE;`

But also, html viewport settings, classes on body-tag, various block classes,
theme suggestions, …

### YML

- **info:**
  - Aliasses: These are used in Twig when you need to include another file.
  Makes it really easy to simplify a path so you don't need to change it in
  a 100 different places if something is renamed
  - global libraries: libraries that are used (almost) everywhere
  - requirements: the modules this theme depends on
  - regions: regions made available for block layout. Beware there are several
  locations to put you navigations in. **Primary and secondary nav are bundled
  together in the mobile navigation.** The rest is not.

- **libraries:**
  - bundles of js (and sometimes css as well). Don't forget you still need to
  attach these to your theme if you want to use them. Either in *.info, in
  *.theme or in a twig file

- **layouts:**
  Templates for Layout Builder (we no longer use DS or Page Manager). At least 1 template in here:
  an example of a custom layout, to use on sections.
  Other Layouts you find in Layout Builder Section configuration, are
  inherited from the rocketship_core module and are in use everywhere Layout Builder is used

- **breakpoints:**
By default, we mostly use the breakpoints defined in rocketship_core. But you can make your own in here


### Components

The themes are built using a component-based (or Atomic) method.
Before going further, it is recommended you get familiar with the idea and
practice behind this. To do so, here are some resources:

- http://bradfrost.com/blog/post/atomic-web-design/
  - More in depth: http://atomicdesign.bradfrost.com/chapter-2/
- Implementation example: https://patternlab.io/
  - Demo: http://demo.patternlab.io/

Our method is slightly different (eg. in order to try and avoid naming
confusion with D8)

First, a distinction about what the components are used for:

- 00-theme
- 01-paragraphs
- 02-features

Inside that is the atomic structure:

- **base**: config and vars
- **atoms**: simplest elements, eg. fields
- **molecules**: groups of atoms, eg. most blocks
- **organisms**: groups of molecules, eg. paragraphs or site regions
- **pages**: Drupal pages + our html frame. This is also where we add the font,
because it works with JS in the HEAD of the HTML
- **layouts**: (custom) layout templates


All of these offer a **combination** of twig, js and css-files.
The reason you will not find a lot of CSS here, is because of the complexity
of our Sass. There are lots of imports (dependencies) and cascades that are
easy to break if we start splitting the CSS.
We do split off the CSS for the Features and Paragraphs from the rest of the
theme.

**Naming** is important for our structure. Sometimes we use numbered folders,
in order to control the loading order. Files starting with and underscore (_)
indicate that they are imported or included somewhere.

**In the future**, there will be integration of an atomic styleguide
(like [Pattern Lab](http://demo.patternlab.io/)). But for now you can find a
drupal **styleguide** in `admin/appearance/styleguide`


### Components vs templates

Because this is a component-driven theme, based on the [Pattern Lab structure](http://patternlab.io/docs/pattern-organization.html), You'll find a bunch of Sass, JS and twig-files together inside the `components/00-theme`-folder.
We've also removed the drupal templates folder and just moved those templates insid the relevant component folder. You can recognize them by the fact they have the `.html.twig` extention and are always nested in a `templates`-folder within a `_drupal` folder.

- The theme's `.info` file, let's you list aliasses in order to include or extend twig-templates (see under: component-libraries).
- That way, we can move the content of every template that is in the `templates`-folder to the `components/00-theme`-folders, creating better-named components
- we also have some extra Twig functions available in there, eg. to make BEM-classes easier to construct. You'll find those in `components/_twig-components`
- to add JS-files per component, you'll have to use drupal's `attach_library()`-function to add libraries to you component's main twig file,
The library name is declared in the theme's `libraries`-file.
The status-component is a good example, you can find the relevant files in `templates/misc/status-messages.html.twig` and `compontent/00-theme/02-molecules/status`

#### Naming conventions

Component files end with `.twig` because we don't need to follow Drupal's templating naming conventions: the shorter, the better. So this naming also helps to separate it from the conventional Drupal templates if they would accidentally share the same name.
The file names start with an underscore to indicate that these files are 'partials' and are meant to be included somewhere (eg. in Drupal templates or in other component files).
Looking at a possible future Pattern Lab integration, this will help exclude the component files from being picked up when generating a html styleguide.

#### BEM

As mentioned above, there is a twig-function that can generate BEM-classes for you.
It looks like this: `{{bem(block, element, [modifier], [extra])}}`
This function essentially outputs the same stuff as {{ attributes }} + outputs a bunch of classes made with the BEM principles if you pass those variables to it. Keep in mind the modifier and extra classes are passed as arrays.
That way you can still have the 'normal' drupal classes as well as BEM classes if you need that.
So the output in html may look something like this:
`class="block__element block__element--modifierA block__element--modifierB extra_class01 extra_class02 default_drupal_class" id="someidhere" role="group"`

Check out `00-theme/example` for an example and more info.

#### Tables

By default, we'd like all tables to be responsive. There are 2 methods in place:
- default, is that the `th` and `td` elements will become blocks on mobile and thus form 1 column.
- the other way, is that the table gets a wrapper with a horizontal scrollbar on mobile

What type of responsive behavior is expressed, depends on what class is present in the table attributes.
By default, there is no special class passed to the attributes, and so the twig template will get wrapped in a div with class `is-reformatted`. This causes responsiveness of the first type.
But if we want another behavior, we can pass a class `table--reformatted` and that will cause the table to be wrapped in a div with class `has-scroll`. This causes responsiveness of the second type.

however, sometimes one method or behavior is not desired. A couple of ways to override this:
- follow the theme suggestions to override the twig template and set the responsive class we want for that new table template
- directly override the table classes or responsive setting in a theme function eg. `$variables['table']['#responsive'] = FALSE;` to turn responsiveness off or use `$variables['classes']` to remove and replace the `table--reformatted` class.


### Config

This is where default values are defined for the theme functions created in
rocketship_theme_flex.theme. You don't need to touch these

### Gulp

We use Gulp as our task manager. This will compile all the things we need to
make the front-end work.

#### + Browser sync

First of all: if you work with Dropsolid's launchpad, gulp can use Browsersync to
automagically update your browser when you change your Sass or JS
This requires you to pass 2 arguments to your gulp tasks:
- projectUrl: a string, the url of your project, including http(s) and the port!
- projectDelay: a boolean, default is FALSE,  if you have a slow sync, you'll
have to tell the system to wait and reload again after the sync has finished.

Example:
`gulp --projectUrl http://rocketshipdemoflex.docker.localhost:81/ --projectDelay`

Your browser will use localhost:3000 to render the site and keep listening to
updates.
It wil **NOT** work if you go to your default project url.

**Extra options:**
If you change the port to 3001, you will get extra development options.
Eg.
- sync across devices: you also get a proxy url so you can surf to your
machine with any device. Very handy if you want to debug on Android or iPhone

**Fixes**:
- If you get a blank page in Chrome or other browsers, chances are localhost isn't getting resolved in Launchpad. [This Firefox/Safari workaround also fixes this issue for Chrome.](https://gitlab.com/dropsolid/launchpad#i-cant-browse-to-a-launchpad-project-in-firefox-or-safari)

#### Linting

It is important that you write your CSS in a structured and clear way.
To enforce this, we've added linting that follows idiomatic CSS and a couple of best-practices.

One of these aspects, is the enforcement of a loose ordering of the properties.
We order the CSS properties by group, but inside those groups you can order the properties in any way you like.

Most properties fall into these groups and thus should be orderened that way:
  - positioning
  - display
  - box model
  - list
  - font
  - background
  - box-shadow
  - transform
  - transition
  - animate

Eg. "font-size" and "font-weight" are part of the same group ('font') so it doesn't matter what goes first.

Eg. "width" is part of the dimensions & box model, so should come before "font-size".

Eg. "border" is part of box model so should go before "box-shadow" and after "font-size".

You can see the full list of properties in `gulp/config.js`, under the section `order/properties-order`

#### + Tasks

Knowing this, you can now use gulp with any of the following tasks.
Enter `gulp` followed by one of the following task names:

**The basic commands:**

- `dev`: same as the default task, this will compile CSS and JS for development
(with linting), try to create favicon and trigger a 'watch' task. 90% of the
time, you only need this task.
- `prod`: compile CSS and JS for production (no map files nor linting nor watch)
- `watch`: included in ‘default’, this task will keep running in the background
to keep compiling the Sass, JS and images
- `favicon`: This will generate the favicons (if you place an image in
/favicons/source folder).  <b>Don't forget to add these images!!!</b>

**What is running in the back of these:**

- `setup`: included in ‘default’, this will check for the favicons but also
try to provide a copy of Fontawesome locally. Please use a CDN for fontawesome
though. This should already be active in html.tpl.php
- `css:dev` and `css:prod`: this will compile the Sass (inside a component
folder) to CSS (and provide sourcemaps if dev is used). The produced files end
up in the `css`-folder
- `js:dev` and `js:prod`: this will concatinate any JS-files inside a
component folder (and provide sourcemaps if dev is used). The produced files
are located in the `js/dest`-folder
- `css:mail`: runs after the css-task, produces css from the style.mail.scss
file
- `css:editor`: runs after the css-task, produces css from the
style.editor.scss file
- `js:lint`: included in ‘js:dev’, this will provide notes for you to write
better code
- `css:lint`: included in ‘css:dev’, the same but for Sass
- `critical`: not currently included, this will help generate a file with
'critical' css for a certain site path. Currently not used but you could use
the produced file to put inline, in the HEAD of your site.
Needs some setup in order to use (see task)

**The extras:**

- `icons:font`: not included in ‘default’, using gulp iconfont this will
generate a font based on svg’s in the 'icons' folder. It will also add
mixins to use them.
- `icons:sprite`: not included in ‘default’, using gulp sprite this will
generate a sprite based on svg’s in the 'icons' folder. It will also add sass
variables and mixins to use the
- `fonts:inline`: Currently not in use. This will attempt to convert all fonts
called using @font-face in style.fonts.css into Base64 and include it into the
css. Can be useful if you absolutely need the fonts to load with the CSS. But
usually you want to avoid using this. Especially since we use a js-solutions
to load fonts.
- `css:lint --lintingOrder=strict`: use this to enforce a very strict order of the properties. By default, the properties are loosely ordered by category only. You can find the groups and their exact properties in `gulp/config.js`
- `css:prod --fontCache false`: not included in 'default' gulp task. When adding this flag to a gulp CSS task,
there will be a timestamp added to the font files url . This is for cache busting the font files.
If you don't add the flag, the cache string won't be a added.
- `css:critical --criticalUrl https://my-site-page-url` --criticalName my_name: This allows you to generate a critical css file based on the provided url (use a dev or staging version WITHOUT htaccess!!!) and named `css/critical-my_name`
The files do not get automatically loaded, this is done by the `critical css` Drupal module so make sure it is enabled and the base directory is set to `/css/critical`.
This module will provide Twig naming suggestions if you have Twig debug enabled. The proposed name of a critical css file, is always based on the node name. For example, the front page needs `css/critical/front.css` or `css/critical/path-front.css`

#### Compile Sass/JS

To simply watch and compile your Sass/JS, you must navigate to your theme
folder (*/docroot/sites/all/themes/custom/dropsolid_base/[child-theme]*) in
your command line and run `gulp`.

#### Favicon Generator

This theme has an automatic favicon generator. Do create your favicons,
add a png to the favicons source folder
(*/docroot/sites/all/themes/custom/dropsolid_base/[child-theme]/favicons/source*).
This png must be at least 250x250 px and a square format. In your command
line, run `gulp favicon` to generate the favicons in the generated folder
(*/docroot/sites/all/themes/custom/dropsolid_base/[child-theme]/favicons/generated*).
You don't have to add these generated favicons manually, there's a function
in template.php that takes care of that for you. Don't move the favicons out
of the generated folder!

#### Custom icon-font or sprite Generator

This theme also has an automatic icon-font generator. Add the svg's to the
icons folder (*/docroot/sites/all/themes/dropsolid_base/icons*) and in your
command line run `gulp icons:font`. This will generate a custom icon-font
in your fonts folder, that you can use in your Sass files.
This will also produce relevant variables and mixins in the components base
folder for you to use to add these icons in your css.
Another option, is to use `gulp icons:sprite`, if you want to use the sprite
option instead of fonts.
Just like with the custom font, it will generate variables and mixins in the
components base folder.

In order to make use of either option, you will need to change variables in:
`components/00-theme/00-base/00-config/00-variables/_vars-base.scss `

For the curious: [the technical details about how the sprite tasks works](https://www.liquidlight.co.uk/blog/creating-svg-sprites-using-gulp-and-sass/)

#### Critical

You'll have to add subtasks per path you want to render critical CSS for.
Make sure these pages have content in them, otherwise it's pretty pointless.
Also, the critical CSS files are useless if you don't inline it in the HEAD
of your site. You can use a module like critical_css to do this.

If you need to inline one yourself, you can do so like this:

```
{{ source( base_path ~ directory ~ '/css/' ~ value)|replace({'../fonts': base_path ~ directory ~ '/fonts'})|raw }}
```

or maybe like this:

```
<style media="screen">
  {% include base_path ~ directory ~ '/css/critical/filename.css' ignore missing %}
</style>```

#### fonts:inline

You probably won't need this. It's for extreme cases where you really need to
have your font loading alongside the CSS. But since we use a decent font loading solution, it's
probably not a good idea.


### Fonts

We use a combination of 'FOUT WITH A CLASS', critical CSS and preload strategies (see sources further down) and fontfaceobserver.

But with a couple of changes to make it more generic for Sass and Twig to handle it automatically.
For example, there is no automatic subsetting of your font(s)
That said, you can tweak and change at your own convenience where needed, if you read up on what is going on (info and sources below).

#### How to use

Just fill in the variables that are in `components/00-theme/04-pages/html/_html.twig`:
- If you have a Google font or Typekit (or any other externally hosted font service), use the google or typekit variables as you see in the examples.

You can reuse those variables for any kind of service if it provides you with a css-file url.
- For any kind of font, always add the font-family name to the `webfont_config_font_families` array variable. This prints classes on the html-tag when the fonts are loaded.
- For self-hosted fonts only: if you want to preload them, add the font-files to `webfont_config_font_files` (path relative to the fonts folder of your theme)
- The `webfont_config_font_types` variable is used to determine what file types to preload. Normally it is only `woff2`, so you don't need to change this.

Some things to keep in mind:
- use self-served fonts when possible. This means: download Google fonts and convert them to woff and woff2 (eg. using font-squirrel)
- if no choice but to use an external service, if they provide a css file that loads the fonts, that's the url you need to put in the google or typekit variables. Otherwise, you'll have to use their method tweak the html twig file(s).

**Exceptions:**
To add an external font to your CKEditor CSS, you will need to use an @import rule into style.editor.scss.
The reason is that we don't want to start overriding the back-end theme in order to load fonts.
Eg: `@import url("https://use.typekit.net/your_typekit_id.css");`

#### How it works

First off, If there are  **self-hosted fonts** (font-files in /fonts), preloading them will shave of some time to try and have the files ready by the time the CSS tries to use them.
This is done in `components/00-theme/04-pages/html/_html--master.twig` using a link-tag and 'preload' settings by default, so the font will not render-block the page.
Anything in the variable `webfont_config_font_files` is used to preload.

**Note:** Do not use with very large fonts or a large amount of fonts. Because if the preload is too big, it can start the initial render of the page. For example, if you have a main text font, a title font and icon fonts, you're stretching the limits. If you see the initial rendering being slow, try removing 1 or 2 fonts from `webfont_config_font_files` and see if it improves.

**Example output:**
```
<link as="font" rel="preload" href="/themes/custom/dropsolid-theme-demo-8/fonts/Montserrat/montserrat-italic-webfont.woff2" type="font/woff2" crossorigin="anonymous" />
```
---
For **externally loaded** fonts, we don't try to load those font-files. Usually there are access restrictions on those and even if there are not, an externally loaded font will not load the same way a self-hosted font will.
Usually you get some code-example that is basically a link-tag  or an @import. We try to turn that into a link-tag which loads the external fonts css-file, using a preload method + asynchronously so it is not render-blocking.
The `webfont_config_google_css` and `webfont_config_typekit_css` variables are used to hold the url to the external css file.

**Note**: No font-files should be loaded with this method. So don't add those font families to `webfont_config_font_files`

**Example code:**
```
<link  as="style"  rel="stylesheet preload prefetch"  href="{{webfont_config_google_css}}"  type="text/css"  crossorigin="anonymous"  onload="this.rel='stylesheet'" />
```
---
If we choose to use **our own fonts css file** (style.fonts.css, which loads our self-hosted fonts like FontAwesome), it is loaded as inline CSS (critical css). This means it is read alongside the html doc being read so it speeds things up quite a bit.
In twig, we replace the paths relative to the fonts folder using a twig filter. That way you can still use the fonts Sass in an intuitive way and it remains workable if you don't put it inline.
Please note the `|raw` filter needing to be used as well, otherwise the quotes in our css file get converted and we don't want that.

**Example code:**
```
<style>{{ source( base_path ~ directory ~ '/css/' ~ value)|replace({'../fonts': base_path ~ directory ~ '/fonts'})|raw }}</style>
```

If for some reason you don't want to go that route, there is an example to load it as a link-tag and still be asynchronous. It is commented out in the master twig file as well:
````
<link rel="preload" href="{{base_path ~ directory ~ '/css/' ~ value}}" as="style" onload="this.onload=null;this.rel='stylesheet'">
<noscript><link rel="stylesheet" href="{{base_path ~ directory ~ '/css/' ~ value}}"></noscript>
<script>
/* You should probably put a polyfill here, like loadCSS, if you care about browsers that don't support preload */
</script>
````
---
We also have a bit of JS (fontfaceobserver) to **detect when our font-families have finished loading** and set classes on the html.
The real strength of this, is that we can make sure our CSS only uses the webfonts when they are really ready to be used, and it can build up the font rendering gradually. So you don't get full FOUT.
So, it is most useful if you have a big, heavy main font you can subset (=use @font-face to only use certain styles/weights, or a limited set of characters).

**Example font-face**
```
@include  font-face((
  font-family: "LatoSubset",
  font-style: normal,
  font-weight: normal,
  file-path: "../fonts/Lato/lato-regular-webfont",
  unicode-range: "U+0100-024F, U+0259, U+1E00-1EFF, U+2020, U+20A0-20AB, U+20AD-20CF, U+2113, U+2C60-2C7F, U+A720-A7FF" // Latin extended
));

@include  font-face((
  font-family: "Lato",
  font-style: normal,
  font-weight: normal,
  file-path: "../fonts/Lato/lato-regular-webfont",
  unicode-range: "U+0100-024F, U+0259, U+1E00-1EFF, U+2020, U+20A0-20AB, U+20AD-20CF, U+2113, U+2C60-2C7F, U+A720-A7FF" // Latin extended
));

@include  font-face((
  font-family: "LatoBold",
  font-style: normal,
  font-weight: bold,
  file-path: "../fonts/Lato/lato-bold-webfont",
  unicode-range: "U+0100-024F, U+0259, U+1E00-1EFF, U+2020, U+20A0-20AB, U+20AD-20CF, U+2113, U+2C60-2C7F, U+A720-A7FF" // Latin extended
));

@include  font-face((
  font-family: "LatoItalic",
  font-style: italic,
  font-weight: normal,
  file-path: "../fonts/Lato/lato-italic-webfont",
  unicode-range: "U+0100-024F, U+0259, U+1E00-1EFF, U+2020, U+20A0-20AB, U+20AD-20CF, U+2113, U+2C60-2C7F, U+A720-A7FF" // Latin extended
));
```
**Example font-families use**
```
body {
  font-family: "LatoSubset", sans-serif;
}

.font-text01-loaded body {
  font-family: "Lato", sans-serif;
}

strong {
  font-family: "LatoSubset", sans-serif;
  font-weight: bold;
}

.font-text01-loaded strong {
  font-family: "LatoBold", sans-serif;
  font-weight: 700;
}
```

**Example JS (in Twig)**
```
var font-latosubset = new FontFaceObserver('LatoSubset');

Promise.all([font-latosubset.load(null, 10000)]).then(function () {

  document.documentElement.className += " font-latosubset-loaded";

  // step 2: load the full fonts
  var fontA = new FontFaceObserver('Lato');
  var fontB = new FontFaceObserver('LatoBold', {
      weight: 700
    });
  var fontC = new FontFaceObserver('LatoItalic', {
      style: 'italic'
    });
  Promise.all([
    fontA.load(null, 10000),
    fontB.load(null, 10000),
    fontC.load(null, 10000)
  ]).then(function () {
    document.documentElement.className += " fonts-loaded";
    // Optimization for Repeat Views
    sessionStorage.fontsLoadedCriticalFoftPolyfill = true;
  });

});
```
---
**Final notes**:

- If you have a lot of fonts or they are very heavy, you may not want to preload the font files.
  - Remove whatever files you don't want to preload, from the `webfont_config_font_files` variable array (if this variable is empty, you can set it to false)
  - Use a subset of your font if possible (read the info about critical FOFT and 2 stage loading + there is an example snippet in `04-pages/html`)
  - Make use of the classes that JS sets on your html: only use your full font-family in CSS, when that class is there. Use the subset font ($fallback-font) by default. if you look for `$fallback-font` in the components Sass, you will see simple examples of its use.
- If you can't/don't want to use the inline css method (critical fonts css) for some reason, there is an example of using an async, preloaded <link>-tag instead. It is located right below the <style>-tag
  - One case could be that your critical css is so big it delays the loading of the doc but that would be exceptional
  - It may be useful in a loading strategy for big font files. You'll have to test the effects yourself.

#### Sources:

- https://slides.com/munter/high-performance-web-fonts-3
- https://www.zachleat.com/web/comprehensive-webfonts/#fout-with-a-class
- https://www.zachleat.com/web/comprehensive-webfonts/#foft%2C-or-fout-with-two-stage-render
- https://www.zachleat.com/web/comprehensive-webfonts/#critical-foft
- https://www.zachleat.com/web/comprehensive-webfonts/#critical-foft-preload
- https://www.zachleat.com/web-fonts/demos/critical-foft-polyfill.html
- https://www.zachleat.com/web/critical-webfonts/
- https://github.com/bramstein/fontfaceobserver/tree/v2.1.0


### Twig

Twig is a powerful templating engine, which allow us to write html around the
Drupal data in an easy way.
Some of the things you use are:
- [**filters**](https://twig.symfony.com/doc/2.x/): so you can manipulate
strings (eg. translate, find/replace stuff, make lowercase, …), split arrays, …
- [**include**](https://twig.symfony.com/doc/2.x/tags/include.html): include a
twig file inside another one + pass variables to it using `with {}`
- [**extends**](https://twig.symfony.com/doc/2.x/tags/extends.html): override
blocks from a master template
- [**embed**](https://twig.symfony.com/doc/2.x/tags/embed.html): combines both
include and extends
- [**macro**](https://twig.symfony.com/doc/2.x/tags/macro.html): kind of like
a function. You reference a macro and pass variables to it.

For Drupal's theme suggestions to work, it needs to have its templates located
in a folder called `templates`. For this reason, we put all our suggested
.html.twig files inside its own `_drupal/templates`-folders BUT do an
`@include` to whatever twig file we control the naming of.

The aliases used with these includes, are defined in rocketship_theme_flex.theme. The
make it easier to build a path.
Combine this with underscores and clear naming for better readability and use
in future pattern styleguide.

There are also some extra twig functions, most notably a BEM function.
This lets you create classes following the BEM-structure for better writing of
CSS (extended Twig-function, thanks to unified twig module)

**Examples:**
- `components/example` naming, embed, aliasses, BEM, …
- `molecules/menus/menu` for include/embed, macro, BEM, …


### Sass

As mentioned before, the styling is combined in the components folder with the
twig and JS. It is also split for Features, Paragraphs and Theme.

- 00-theme
- 01-paragraphs
- 02-features

And inside each of those:

- **base**: config and vars
- **atoms**: styling the simplest elements, eg. fields
- **molecules**: styling groups of atoms, eg. most blocks
- **organisms**: styling groups of molecules, eg. paragraphs or site regions
- **pages**: styling Drupal pages (eg. 404, user, maintenance, …)
- **layouts**: styling for (custom) layout templates

All theme files combine into style files (with @import) for specidic perposes:
admin, editor, fonts, mail, print, …

Only compiled CSS-files loaded in a library (eg. in .info file, a hook or a
twig-file)

### JS

Folowing the same atomic structure, you will find javascript files for
specific functionality added to specific components. Most of these are **loaded
selectively**.
Eg. language menu JS is set in `molecules/links/links--language` (makes a
dropdown menu or shortcodes) or dropdown menu and mobile menu JS is set in
`molecules/menus/menu`).

There are also 2 special JS files, located inside `00-base`, these give you
some helper functions and global functionality. These are **loaded globally**,
alongside jQuery, jQuery Once, drupal JS, drupalSettings, ….

All js-files located in the atoms/molecules/organisms/… folders will compile
into `js/dest`.

If you make a new component:
1. add your original files to your new component's folder
2. include the generated file and not the original source to your library
3. don't forget to add your dependencies (eg. jQuery, …). Even if it is already
added globally, this makes sure they are loaded BEFORE your JS is run
4. attach your library to your component's twig-file if possible. If you don't
have a relevant twig-file to add it to, there are other ways to load the libary
(see previously in the readme)

!!! DO NOT ADD YOUR JS-FILES TO THE 00-BASE FOLDER !!!

**Namespaces:**
We use a namespace (basically a global variable) to save all our functions in.
This is to make sure that we can easily reference functions/variables defined
in a separate file and make such references readable. This would not be a big
problem with aggregated files but would be during development.
See the various JS-files to see what I'm talking about.

**Behaviors**
Drupal behaviors are great. Look them up sometime.
Bundle your js triggers inside a behavior to ensure they are loaded after Ajax
calls.
You can split off your actual JS functions and put them outside the behaviors.

**Bigpipe**
There is a link to documentation about Bigpipe in the Sources at the end. In
short: Bigpipe is a system that loads chunks of the drupal site separately for
better loading efficiency. The language navigation is one of those elements
that is handled by Ajax in that way. That's why you don't see the active class
or property printed out in the Twig file.

**jQuery Once**
Examples of how to use this are in the top comments of all JS files.
These help to avoid loading JS (eg. event handlers) multiple times.

**WARNINGS:**

- There is a bug in Drupal 8.6 that breaks the 'context' property when you try
to get an element, in a library that is not loaded globally in your info file.
E.g. `$('.myclass', context)` gives back an empty object (length 0), but
`$('.myclass')` works.


### Fonts

We use Google/Webkit's font loader in order to render non-blocking fonts. This
is done using a bit of JS included in the HEAD of your site.

**Setup:**

- `fonts`-folder to contain your font-files, if not using google fonts or
typekit
- `00-base/02-fonts`: reference your font files, if not using google fonts
or typekit
- `00-base/00-config/_vars-01-typo.scss`: variables for your font family names
- `01-atoms/_00-normalize.scss`: style a preloader if you want. By default it
is turned off. By turning it on, your page fades in after fonts are done
loading.
- `04-pages/html/_html.twig`: settings for google/webkit/custom webfonts.
Fontawesome is already included, add custom iconfont here as well, if you have
one


## Styleguide

As mentioned before, there will be integration of an atomic styleguide at some
point. For now you can find a drupal styleguide in `admin/appearance/styleguide`

This uses the `styleguide` contrib module as a starting point and extended with a
rocketship_styleguide module, we take over the page and generate our own elements in the
order and groups we want them.

Using this, you should be able to style most of the atoms and part of the
molecules (eg. pager).

## Breakpoints

Breakpoints are set and used in 3 places:
- **in the theme (and dropsolid_core)**
  `breakpoints.yml`-files:
  These are used for responsive image styles by Drupal
- **in the Sass**
  `00-theme/00-base/02-helpers/02-mixins/_mixins-00-breakpoints.scss`
  `00-theme/00-base/00-config/00-variables/_vars-breakpoints.scss`
  You get a lot of possible breakpoints here but the most important thing to
  remember, is that we work from the 'mobile first' principle.
  **Meaning**: most breakpoints go from smaller to bigger screens using
  'min-width'.
  **However**, there are exceptions: we have breakpoints to add styling
  specific for a certain breakpoint (eg. bp-xs-only is only small phones)
  OR smaller than a max-width (eg. bp-sm-only is for screens smaller then or
  equal to tablet).
  **Why do we have those exceptions?** Because sometimes a mobile design is
  very different from the bigger screens (eg. mobile menu and navigation) and
  we don't want to have to undo a bunch of styling.

- **in the JS**
  `00-base/06-scripts/00-helpers.js`
  There is a `self.screen` variable set up so you can check on which breakpoint
  you are. This is used by the JS that does the mobile menu navigation but can
  come in handy in other places as well.
  It relies on the getBreakpoint and checkScreensize functions, which refer to
  a pseudo-element set by CSS.



## Special components

### Forms

We use floating labels in the forms.
`_forms--helpers.scss` defines the styling in mixins, either on a group of
labels or individual label.
`_forms.scss` calls the mixins for groups of labels in specific forms, or
individual labels.
`forms.js` builds classes for the styling to work, on user interaction.

We also mark fields as being 'optional' instead of marking the ones that are
'required'. The styling and twig for this element is located in the
`form-label`-folder.
If you need to replace with 'required' fields, you can remove that element in
the Twig-file or hide it in the Sass, and style the other element instead.

### Header & navigation

The header can get quite complex, with several menu's and blocks added to it.
Especially because the mobile navigation can include different elements than
the bigger menu and certain blocks need to be displayed in different places
depending on the breakpoint.

**Add your blocks and menu's to the right regions:**

- **header_top**: blocks and stuff that goes in a bar above the primary header
& logo. Eg. language switcher, contact info or social media icons
- **top navigation**: menu's that are in the header top (so above the prim.
header)
- **primary header**: usually only the logo (branding block) is in here. Unless
you need some blocks besides whatever is in the navigation.
- **nav primary and nav secondary**: these 2 regions are used to build the 2
possible navigations that go next to or under the logo in the header. Those 2
are grouped into the 'nav' (twig files) and 'mobile navigation' (concept).
So anything in those regions is visible in the open mobile menu. Eg. the
primary and main menu's + any CTA blocks next to it. Also social media icons
that are visible on mobile but not desktop.

As you can read, we have some specialized groupings in place for building the
elements of the header and navigation. Of course these all need to be styled
in the right place as well.

**Be careful to add your styling to the right files:**

- styling the search block: molecules/search/search-block
- styling the other blocks: molecules/whateverblock
- styling the menu's that go inside the navigation: molecules/menus/menu
- styling the header logo: molecules/site/logo
- positioning the elements of the navigation (menu's + blocks):
organisms/header/nav
- positioning the elements in the header (logo + several wrappers):
organisms/header/header--primary

As mentioned in the `breakpoints`-section, the navigation uses breakpoints in
order to make a mobile navigation and a bigger navigation.
We also use several block layout regions to group elements from the header and
navigations in a way that works for mobile as well as wider screens

## Layout 01: Twig

Look at a drupal site as nested dolls:

- **block layout**: the main frame of our site. These contain all the regions
set in the Drupal html twig file
- **panels**: [no longer used] The making of sub-regions inside the 'Content' region of the 'block layout'.
You could make layout templates and define those sub-regions using
**Page Manager** [no longer used]. You could do so for different types of pages (eg. for different
content types like Landing Page, Blog detail, …). But as said, is no longer in use since we use Layout Builder.
- **view-modes**: the collection of fields and stuff that builds up a node (Block, Paragraph or Content Type) of a certain type (eg. Blog Content Type). There are view-modes created per context. Eg. a teaser may not need to contain all the same fields as the full node.
You can use Layout Builder to make a structure using rows (Sections) combined with layout templates. Inside those layouts, are Regions, which are basically the columns that contain the fields (and optionally, custom blocks)
All this can be found under the `manage view modes` tab of a Block, Paragraph or CT
**Display Suite**. [no longer used] We used to use this instead of Layout Builder. You could also use layout templates (via a dropdown at the bottom of the view-mode page) but also make groups via the UI (html-tag with optional
classes).
- **node variants**: When Layout Builder is active on a Content type, you can actually specify to use a different layout ONLY on a specific node instead of on ALL the nodes of that Content Type. This way you can make landing pages. The homepage is a current example of this. It's a basic page that has been changed.
- **theme twig files**: all entities (fields, blocks, ct's, paragraphs, …)
have corresponsing Twig-files (you can see their names and paths when theme
suggestions are visible in html). The twig-files for view modes (only in blocks,
ct's and paragraphs) can be overridden with layout templates, if Layout Builder has applied one.

**Fields:**

field don't HAVE view-modes, they are rendered INSIDE view modes. You do have
some tricks to change their output without dealing with Twig though. Look at the
formatters and options in the various view modes. Formatters can also be custom
made, but that's back-end dev stuff and outside the scope of this documentation.

**Layout Builder vs default twig files:**

So for blocks/ct's/paragraphs, you have a choice to either use the default twig
templates and have full control of the output inside those (no predefined
subregions, no UI to help you). Or you can avoid this manual work by using
layout templates where regions can be pre-defined. For Flex projects, it is recommended to
use the provided layouts and their regions.

**Naming files:**

When making your own layout templates, keep in mind that **the machine name
has to be the same as the twig file name**. The machine name uses underscores
(_), the twig file uses dashes (-).


### Example: Homepage

Rocketship content types use Layout Builder to build up Sections (rows) and regions in 1 to 3 columns, to output node
content. For the homepage, different layout templates from Rocketship Core are used but you can make and add your own.
An example is provided in the theme/05-layout folder.

- `00-theme/05-layouts`: where any extra custom layout templates should be located.
Add your regions and html here and special Section/Region styling here.
- `rocketship_theme_flex.layouts.yml`: The regions are defined here for custom layouts
- `go to homepage > layout tab`: See a list of all current Sections, where you can add and configurate your own. You will need to name your section but also add a custom class in order to differentiate the sections from each other.
Please not that you can not switch layouts once a Section has been created. So if you want to use another layout instead, you need to add a new Section, apply your preferred layout, and name it.

### Example: Blog complex

When using Rocketship (or Dropsolid) Features (not publicly available yet, as of this writing):

You will see that most Rocketship contenttypes have at least 2 view-modes they use:
- teaser: for the main overview
- full content: for the node detail content

When click the Layout Builder button on the full content node, you will see that different sections make up different parts of the node.
At the very least, you should see a section that contains the 'head' of your node and one for the 'content'. Both should have custom classes in order to better style them.
The naming follows the BEM rules for CSS.


## Layout 02: CSS

### Vertical rhythm

In order to set up consistent vertical space (vertical rhythm) and horizontal
spacing, the theme contains several variables in
`00-theme/00-base/00-config/00-variables/_vars-01-layout.scss`
You can tweak these if you need, but keep in mind they are part of a system
used throughout the whole site to make your life easier. If you just need 1
element to have a different amount of spacing (eg. the homepage blocks and
rows), please just use another variable in those elements.

- `$gap-vert` is based on the line-height, because this will make space in
texts (eg paragraphs and lists). Therefor, it is based on the typography
sizing. It's the basis used in print design for ensuring readability and a
nice vertical rhythm.
- `$gap-vert-xs` `$gap-vert-sm`: these are 2 smaller variants of this, used
when less space is needed. Eg. with small titles or other smaller bits of text.
The reason these have a fixed size, is because you can't set a minimum margin
or padding size, so we had to settle for a fixed value instead. So you might
need to tweak these a bit based on how big your line-height is.
- `$gap-vert-lg`: The bigger gaps are still based on your typography

Besides those, we also have special top and bottom spaces, per breakpoint.
These are used by almost all rows of content. Most notably all the paragraphs,
headers, related views, …
`$gap-vert-top-…` and `$gap-vert-bottom-…`

### Horizontal space

There is a default gap size defined, that is also the fallback for the grid
system gaps between columns. Try to keep this value the same as in the designs.
The container padding is half of the columns gap.

### Positioning layers

That layout variables file also contains a map of z-indexes you can use to
keep overview of all the layers in your site. This is especially important
for the mobile menu and other overlays.

!!!USE THE MIXIN AND THESE VARIABLES!!!

**Important!**

- Keep in mind that a z-index ONLY works on an element with a position
(relative/absolute/fixed/…).
- Remember that if you nest layers, their z-index is always RELATIVE to the
closest wrapper with a position.
- For these reasons, the `.container` and several other wrappers do not have a
default position: relative. We want to avoid having to override these in the
header.
- Resetting positioning on an element can always be done by giving it
`position: static`. This makes the element ignore position, z-index,
left/right/top/bottom.

### Grids

`00-theme/00-base/05-grid`
We have a grid system in place that uses flexbox and has fallback stuff for IE.
Inside this folder, you will find the mixins and examples using html.

You might also notice there is a `_alt` folder. This one contains a couple of
experimental grid systems, using CSS Grid. Do not use this in production as
long as IE is supported. The fallback system is complex and unstable.

## Features

Only important if using Rocketship (or Dropsolid) Features:

Important to know for theming, is that these create contenttypes which have view modes with layout templates and classes for you to theme them.

The styling is located in a separate folder of the components: 02-features

### Layout Builder

- Explained earlier, with example from the homepage

### Overviews

See the info in the 'development' readme of your Rocketship project, for making your own views, how to add them to the homepage or adding them to a Page.

### Details

As mentioned before, the Layout Builder page for this loads the entity content's
view mode 'full' or 'full content'.

### Libraries & styling

The Features and Paragraphs have libraries inside their modules, these contain
functional JS (eg. FAQ paragraph needs to open and close with a click).
They also load a structural CSS-file.

If you have a rocketship theme enabled, each Feature and Paragraphs
loads styling from this theme, when the Dropsolid Feature of Paragraphs modules are enabled.

This is done by attaching a library via the theme's `rocketship_theme_flex.theme` file's `HOOK_preprocess_page`

The styling can be changed in the `component/02-features` and `component/01-paragraphs` files

## Paragraphs

### Background color & image

- **rocketship_theme_flex.theme**:
  `HOOK_rocketship_paragraphs_bg_color_options_list`
  This is where you need to add the extra colors that the simple header
  and other paragraphs use for the front-facing theme.
- `00-theme/admin.style.scss`
  You don't need to touch this file to work on paragraphs
- `components/01-paragraphs/03-organisms/_paragraphs.scss`
  define the background colors and related styling for the paragraphs in the front-facing theme
- `components/01-paragraphs/03-organisms/p015-simple-header`
   `components/01-paragraphs/03-organisms/p016-focus-header`
  define the background colors and related styling for the headers,
  as used in Page content type, Office, Product, …

In order for the background-colors preview to work in the admin theme, you need to fill in your color variants at `/admin/config/system/rocketship-paragraphs/settings`.

**Note:**
- Saving this form will also a default colors CSS-file that is loaded if you check the `Enable default CSS` options. Dropsolid sites DO NOT USE the default CSS, so you still need to add your front-facing colors & other theming in the `03-paragraphs` folder of your theme.

More info on how this all works, can be found in the Rocketship Paragraphs module README file

### Libraries & styling

Same stuff as for the Features

## CKE

### Webfonts
In order to use Google (or Typekit) fonts in the editor, you will need to
@import your font into `style.editor.scss`.
This is because the admin-theme does not use Webfont loader at this point.

DO NOT USE @IMPORT IN THE FRONT-END THEME !!! Always use the webloader in
04-pages/html/_html.twig for the front-end theme.

Example for using Open Sans and Roboto Condensed:
<code>@import url(https://fonts.googleapis.com/css?family=Open+Sans:300,400,700|Roboto+Condensed:400,700);
</code>

### Styles
Just like in D7, we can add Styles. This is done in the UI:
`/admin/config/content/formats`

For each of your editors, you can scroll to the Styles dropdown section
and add your custom styles to it.
Always start with a tag-name and add the class separated by a dot (.)
For multiple variants, you can add all possible classes to your tag, separated
by a pipe (|)
There is a bug though. When you turn on 'limit html tags', the Styles dropdown
breaks.



## Sources

- [Vardumper](https://www.drupal.org/project/twig_vardumper)
- [Twig xdebug](https://www.drupal.org/project/twig_xdebug)
- [Devel: webprofiler](https://www.drupal.org/project/devel)
- [Atomic design](http://bradfrost.com/blog/post/atomic-web-design/)
- [Atomic design: book](http://atomicdesign.bradfrost.com/table-of-contents/)
- [Pattern lab demo](http://demo.patternlab.io/)
- [Drupal 8 documentation (incomplete)](https://www.drupal.org/docs/8)
- [Adding libaries to a theme (CSS, JS)](https://www.drupal.org/docs/8/theming/adding-stylesheets-css-and-javascript-js-to-a-drupal-8-theme)
- [Behaviors](https://sqndr.github.io/d8-theming-guide/javascript/behaviors.html)
- [Behaviors [Bigpipe]](https://www.drupal.org/docs/8/core/modules/bigpipe/overview)
- [API overview](https://www.drupal.org/docs/8/api/javascript-api/javascript-api-overview)
- [Powerful templating engine](https://twig.symfony.com/doc/2.x/)


## Issues

### Theme suggestions don't work with Layouts

For some reason, the theme suggestions don't work properly with the layouts if the template file name (using dashes) is not the same as the template machine (using underscores) name in `x.layouts.yml`.
So, rename the template file name to match.

## Maintenance

As long as the project is used internally only (Dropsolid), merges with the master must be approved by R&D. Use our ticket system to report issues and propose solutions.
If new types of favicons are introduced in the future, they need to be added to the base theme's html twig file + the favicon gulp task

At the time this project moves to drupal.org, issues and patches can be added there.

- Git repos: see the git repo for rocketship_theme_generator module
